/***************************************************************************
*
*	File name : boot.c
*	Function  : Boot function
*	Revision  :
*		07/31/2007	Nariai		First revision
*
****************************************************************************/

// Include File
#include	<string.h>
#include	<libstdio.h>
#include	"reg_801.h"
#include	"system.h"
#include	"flashc_api.h"
#include	"cmu_api.h"
#include	"gpio_api.h"
#include	"sramc_api.h"

// Prototype
void Boot(void);
static void FlashcInitialize(void);
static void CmuInitialize(void);
static void PortInitialize(void);
static void SramcInitialize(void);
static void ClearBssSec(void);
static void LoadDataSec(void);

// Extern
extern int main(void);


/***************************************************************************
	Boot

	description	: boot function
	argument	:
	return: 	:

	input		:
	output		:

	flag		:
	global		:
***************************************************************************/
void Boot(void){

	asm("xld.a	%r0,	0x000000	");

	// Claer IRAM
	asm("xld.a	%r1,	0x000040	");
	asm("xld.a	%r2,	0x000700	");
	asm("CLEAR_IRAM:				");
	asm("ld		[%r1]+,	%r0			");
	asm("cmp	%r1,	%r2			");
	asm("jrne   CLEAR_IRAM");

	// Clear Register
	asm("ld.a	%r1,	%r0			");
	asm("ld.a	%r2,	%r0			");
	asm("ld.a	%r3,	%r0			");
	asm("ld.a	%r4,	%r0			");
	asm("ld.a	%r5,	%r0			");
	asm("ld.a	%r6,	%r0			");
	asm("ld.a	%r7,	%r0			");

	// Set Stack Pointer
	asm("xld.a	%r0,	0x001000	");
	asm("ld.a	%sp,	%r0			");

	// Init. peripheral circuit
	FlashcInitialize();
	CmuInitialize();
	PortInitialize();
	SramcInitialize();

	// Init. ".bss" and ".data" section
	ClearBssSec();
	LoadDataSec();

	_init_sys();                // Initialize for sys.c
	_init_lib();                // Initialize for lib.c

	// Jump main function
	main();

	// End program
	while(1);

}

/* ----- static ----- */

/***************************************************************************
	FlashcInitialize

	description	: Init. FLASHC
	argument	:
	return: 	:

	input		:
	output		:

	flag		:
	global		:
***************************************************************************/
static void FlashcInitialize(void){

	T_FLASHC_CFG	config;

	config.flashWait = 0;			// Access wait value
	// Not change for TTBR
	config.TTBR = 0xFFFFFFFF;		// Trap Table base address

	FlashcInit(&config);

	return;

}


/***************************************************************************
	CmuInitialize

	description	: Init. CMU
	argument	: None
	return: 	: None

	input		:
	output		:

	flag		: None
	global		: None
***************************************************************************/
static void CmuInitialize(void){

	T_CMU_CFG		cmuConfig;

	cmuConfig.osc3Nf		= CMU_NF_ENABLE;
	cmuConfig.inPortNf		= CMU_NF_ENABLE;
	cmuConfig.dsioNf		= CMU_NF_ENABLE;

	cmuConfig.osc3ClockDiv	= CMU_OSC3_CLK_DIV_1_1;
	cmuConfig.lcdcClockDiv	= CMU_LCDC_CLK_DIV_1_16;
	cmuConfig.cmuClockSel	= CMU_CMU_CLK_OSC3;

	cmuConfig.gatedClk0.bit.pclk		= CMU_GATED_CLOCK_ON;
	cmuConfig.gatedClk0.bit.lcdc		= CMU_GATED_CLOCK_OFF;
	cmuConfig.gatedClk0.bit.lcdcSapb	= CMU_GATED_CLOCK_OFF;
	cmuConfig.gatedClk0.bit.lcdcHostIf	= CMU_GATED_CLOCK_OFF;
	cmuConfig.gatedClk0.bit.usb			= CMU_GATED_CLOCK_OFF;
	cmuConfig.gatedClk0.bit.usbSapb		= CMU_GATED_CLOCK_OFF;
	cmuConfig.gatedClk0.bit.flashc		= CMU_GATED_CLOCK_ON;
	cmuConfig.gatedClk1.bit.mft			= CMU_GATED_CLOCK_OFF;
	cmuConfig.gatedClk1.bit.t8			= CMU_GATED_CLOCK_OFF;
	cmuConfig.gatedClk1.bit.sramc		= CMU_GATED_CLOCK_ON;
	cmuConfig.gatedClk2.bit.rtcSapb		= CMU_GATED_CLOCK_OFF;
	cmuConfig.gatedClk2.bit.port		= CMU_GATED_CLOCK_ON;
	cmuConfig.gatedClk2.bit.wdt			= CMU_GATED_CLOCK_OFF;
	cmuConfig.gatedClk2.bit.adc			= CMU_GATED_CLOCK_OFF;
	cmuConfig.gatedClk2.bit.remc		= CMU_GATED_CLOCK_OFF;
	cmuConfig.gatedClk2.bit.spi			= CMU_GATED_CLOCK_OFF;

	cmuConfig.usbWait		= 3;
	cmuConfig.usbSnooze		= CMU_USB_SNOOZE_DISABLE;

	CmuInit(&cmuConfig);

	return;
}

/***************************************************************************
	PortInitialize

	description	: Init. Port
	argument	: None
	return: 	: None

	input		:
	output		:

	flag		: None
	global		: None
***************************************************************************/
static void PortInitialize(void){

	U_GPIO_FUNC_SEL	portFunc;

	// Setting port 6
	portFunc.CFP		= GpioGetPortFunc(6);
	portFunc.b.CFPx0	= GPIO_FUNCTION_1;
	portFunc.b.CFPx1	= GPIO_FUNCTION_1;
	portFunc.b.CFPx2	= GPIO_FUNCTION_1;
	portFunc.b.CFPx3	= GPIO_FUNCTION_1;
	portFunc.b.CFPx4	= GPIO_FUNCTION_1;
	portFunc.b.CFPx5	= GPIO_FUNCTION_1;
	portFunc.b.CFPx6	= GPIO_FUNCTION_1;
	portFunc.b.CFPx7	= GPIO_FUNCTION_1;
	GpioSetPortFunc(6, &portFunc);

	// Setting port 7
	portFunc.CFP		= GpioGetPortFunc(7);
	portFunc.b.CFPx0	= GPIO_FUNCTION_1;
	portFunc.b.CFPx1	= GPIO_FUNCTION_1;
	portFunc.b.CFPx2	= GPIO_FUNCTION_1;
	portFunc.b.CFPx3	= GPIO_FUNCTION_1;
	portFunc.b.CFPx4	= GPIO_FUNCTION_1;
	portFunc.b.CFPx5	= GPIO_FUNCTION_1;
	portFunc.b.CFPx6	= GPIO_FUNCTION_1;
	portFunc.b.CFPx7	= GPIO_FUNCTION_1;
	GpioSetPortFunc(7, &portFunc);

	// Setting port 8
	portFunc.CFP		= GpioGetPortFunc(8);
	portFunc.b.CFPx0	= GPIO_FUNCTION_1;
	portFunc.b.CFPx1	= GPIO_FUNCTION_1;
	portFunc.b.CFPx2	= GPIO_FUNCTION_1;
	portFunc.b.CFPx3	= GPIO_FUNCTION_1;
	portFunc.b.CFPx4	= GPIO_FUNCTION_1;
	portFunc.b.CFPx5	= GPIO_FUNCTION_1;
	portFunc.b.CFPx6	= GPIO_FUNCTION_1;
	portFunc.b.CFPx7	= GPIO_FUNCTION_1;
	GpioSetPortFunc(8, &portFunc);

	// Setting port 9
	portFunc.CFP		= GpioGetPortFunc(9);
	portFunc.b.CFPx0	= GPIO_FUNCTION_1;
	portFunc.b.CFPx1	= GPIO_FUNCTION_1;
	portFunc.b.CFPx2	= GPIO_FUNCTION_1;
	portFunc.b.CFPx3	= GPIO_FUNCTION_1;
	portFunc.b.CFPx4	= GPIO_FUNCTION_1;
	portFunc.b.CFPx5	= GPIO_FUNCTION_1;
	portFunc.b.CFPx6	= GPIO_FUNCTION_1;
	portFunc.b.CFPx7	= GPIO_FUNCTION_1;
	GpioSetPortFunc(9, &portFunc);

	// Setting port A
	portFunc.CFP		= GpioGetPortFunc(A);
	portFunc.b.CFPx0	= GPIO_FUNCTION_1;
	portFunc.b.CFPx1	= GPIO_FUNCTION_1;
	portFunc.b.CFPx2	= GPIO_FUNCTION_1;
	portFunc.b.CFPx3	= GPIO_FUNCTION_1;
	portFunc.b.CFPx4	= GPIO_FUNCTION_1;
	portFunc.b.CFPx5	= GPIO_FUNCTION_1;
	portFunc.b.CFPx6	= GPIO_FUNCTION_1;
	portFunc.b.CFPx7	= GPIO_FUNCTION_1;
	GpioSetPortFunc(A, &portFunc);

	// Setting port B
	portFunc.CFP		= GpioGetPortFunc(B);
	portFunc.b.CFPx0	= GPIO_FUNCTION_1;
	portFunc.b.CFPx1	= GPIO_FUNCTION_1;
	portFunc.b.CFPx2	= GPIO_FUNCTION_1;
	portFunc.b.CFPx3	= GPIO_FUNCTION_1;
	portFunc.b.CFPx4	= GPIO_FUNCTION_1;
	portFunc.b.CFPx5	= GPIO_FUNCTION_1;
	portFunc.b.CFPx6	= GPIO_FUNCTION_1;
	portFunc.b.CFPx7	= GPIO_FUNCTION_1;
	GpioSetPortFunc(B, &portFunc);

}

/***************************************************************************
	SramcInitialize

	description	: Init. SRAMC
	argument	: None
	return: 	: None

	input		:
	output		:

	flag		: None
	global		: None
***************************************************************************/
static void SramcInitialize(void){

	T_SRAMC_CFG		sramcConfig;

	sramcConfig.ce0Wait		= 2;
	sramcConfig.ce1Wait		= 2;
	sramcConfig.ce2Wait		= 15;
	sramcConfig.ce3Wait		= 15;
	sramcConfig.ce0DevSize	= SRAMC_DEV_SIZE_16BIT;
	sramcConfig.ce1DevSize	= SRAMC_DEV_SIZE_16BIT;
	sramcConfig.ce2DevSize	= SRAMC_DEV_SIZE_8BIT;
	sramcConfig.ce3DevSize	= SRAMC_DEV_SIZE_8BIT;
	sramcConfig.ce0MemType	= SRAMC_MEM_TYPE_BSL;
	sramcConfig.ce1MemType	= SRAMC_MEM_TYPE_BSL;
	sramcConfig.ce2MemType	= SRAMC_MEM_TYPE_A0;
	sramcConfig.ce3MemType	= SRAMC_MEM_TYPE_A0;
	sramcConfig.selEvram	= SRAMC_EVRAM_CE2;
	sramcConfig.iramWait	= SRAMC_IRAM_WAIT_1CYCLE;
	sramcConfig.macWait		= SRAMC_MAC_WAIT_1CYCLE;
	sramcConfig.rtcWait		= 5;

	SramcInit(&sramcConfig);

	return;

}

/***************************************************************************
	ClearBssSec

	description	: Clear .bss section
	argument	: None
	return: 	: None

	input		:
	output		:

	flag		: None
	global		: None
***************************************************************************/
static void ClearBssSec(void){

	extern unsigned long	__START_bss;
	extern unsigned long	__END_bss;
	unsigned char			*pClrAddr;
	unsigned long			bssSize;
	unsigned long			i;

	pClrAddr	= (unsigned char *)&__START_bss;
	bssSize		= (unsigned char *)&__END_bss - (unsigned char *)&__START_bss;

	// Clear .bss section
	for(i = 0 ; i < bssSize ; i++){
		*pClrAddr	= 0x00;
		pClrAddr++;

	}

	return;

}


/***************************************************************************
	LoadDataSec

	description	: Load .data section
	argument	: None
	return: 	: None

	input		:
	output		:

	flag		: None
	global		: None
***************************************************************************/
static void LoadDataSec(void){

	extern unsigned long	__START_data;
	extern unsigned long	__END_data;
	extern unsigned long	__START_data_lma;
	unsigned char			*pSrc;
	unsigned char			*pDst;
	unsigned long			dataSize;
	unsigned long			i;

	pSrc		= (unsigned char *)&__START_data_lma;
	pDst		= (unsigned char *)&__START_data;
	dataSize	= (unsigned char *)&__END_data - (unsigned char *)&__START_data;

	// Load ".data" section
	for(i = 0 ; i < dataSize ; i++){
		*pDst	= *pSrc;
		pSrc++;
		pDst++;

	}

	return;

}

