/***************************************************************************
*
*	File name	: cmu_api.h
*	Function	: APIs for CMU
*	Revision	:
*		10/03/2007	Nariai		First revision
*
****************************************************************************/
#ifndef	__CMU_API_H__
#define	__CMU_API_H__

// OSC3 on/off
#define	CMU_OSC3_OFF			0
#define	CMU_OSC3_ON				1

// OSC clock select
#define	CMU_CLOCK_SELECT_OSC3	0
#define	CMU_CLOCK_SELECT_OSC1	1

// Wakeup wait
#define	CMU_WAKEUP_NO_WAIT		0
#define	CMU_WAKEUP_INT_WAIT		1

// Wait timer mode
#define	CMU_WAIT_TIMER_NORMAL	0
#define	CMU_WAIT_TIMER_HIGH		1

// OSC3 disable during sleep
#define	CMU_OSC3_RUN			0
#define	CMU_OSC3_STOP			1

// Noise filter enable/disable
#define	CMU_NF_ENABLE			0
#define	CMU_NF_DISABLE			1

// OSC3 clock divider
#define	CMU_OSC3_CLK_DIV_1_1	0x0		// OSC3 * 1/1
#define	CMU_OSC3_CLK_DIV_1_2	0x1		// OSC3 * 1/2
#define	CMU_OSC3_CLK_DIV_1_4	0x2		// OSC3 * 1/4
#define	CMU_OSC3_CLK_DIV_1_8	0x3		// OSC3 * 1/8
#define	CMU_OSC3_CLK_DIV_1_16	0x4		// OSC3 * 1/16
#define	CMU_OSC3_CLK_DIV_1_32	0x5		// OSC3 * 1/32

// LCDC clock divider
#define	CMU_LCDC_CLK_DIV_1_1	0x0		// OSC3 * 1/1
#define	CMU_LCDC_CLK_DIV_1_2	0x1		// OSC3 * 1/2
#define	CMU_LCDC_CLK_DIV_1_3	0x2		// OSC3 * 1/3
#define	CMU_LCDC_CLK_DIV_1_4	0x3		// OSC3 * 1/4
#define	CMU_LCDC_CLK_DIV_1_5	0x4		// OSC3 * 1/5
#define	CMU_LCDC_CLK_DIV_1_6	0x5		// OSC3 * 1/6
#define	CMU_LCDC_CLK_DIV_1_7	0x6		// OSC3 * 1/7
#define	CMU_LCDC_CLK_DIV_1_8	0x7		// OSC3 * 1/8
#define	CMU_LCDC_CLK_DIV_1_9	0x8		// OSC3 * 1/9
#define	CMU_LCDC_CLK_DIV_1_10	0x9		// OSC3 * 1/10
#define	CMU_LCDC_CLK_DIV_1_11	0xA		// OSC3 * 1/11
#define	CMU_LCDC_CLK_DIV_1_12	0xB		// OSC3 * 1/12
#define	CMU_LCDC_CLK_DIV_1_13	0xC		// OSC3 * 1/13
#define	CMU_LCDC_CLK_DIV_1_14	0xD		// OSC3 * 1/14
#define	CMU_LCDC_CLK_DIV_1_15	0xE		// OSC3 * 1/15
#define	CMU_LCDC_CLK_DIV_1_16	0xF		// OSC3 * 1/16

// "CMU_CLK" select
#define	CMU_CMU_CLK_OSC3		0x0		// OSC3
#define	CMU_CMU_CLK_OSC1		0x1		// OSC1
#define	CMU_CMU_CLK_BCLK		0x2		// BCLK
#define	CMU_CMU_CLK_LCDC		0x3		// LCDC_CLK
#define	CMU_CMU_CLK_OSC3_1_1	0x4		// OSC3 * 1/1
#define	CMU_CMU_CLK_OSC3_1_2	0x5		// OSC3 * 1/2
#define	CMU_CMU_CLK_OSC3_1_4	0x6		// OSC3 * 1/4
#define	CMU_CMU_CLK_OSC3_1_8	0x7		// OSC3 * 1/8
#define	CMU_CMU_CLK_OSC3_1_16	0x8		// OSC3 * 1/16
#define	CMU_CMU_CLK_OSC3_1_32	0x9		// OSC3 * 1/32
#define	CMU_CMU_CLK_SRAMC		0xA		// SRAMC_CLK
#define	CMU_CMU_CLK_USB			0xB		// USB_CLK
#define	CMU_CMU_CLK_OSC8		0xC		// OSC8

// Gated clock on/off
#define	CMU_GATED_CLOCK_OFF		0
#define	CMU_GATED_CLOCK_ON		1

// USB snooze enable/disable
#define	CMU_USB_SNOOZE_DISABLE	0
#define	CMU_USB_SNOOZE_ENABLE	1


// Gated clock 0
typedef union {
	unsigned char		reg;
	struct {
		unsigned char	pclk		: 1;
		unsigned char	lcdc		: 1;
		unsigned char	lcdcSapb	: 1;
		unsigned char	lcdcHostIf	: 1;
		unsigned char	usb			: 1;
		unsigned char	usbSapb		: 1;
		unsigned char	reserved	: 1;
		unsigned char	flashc		: 1;
	} bit;
} U_CMU_GATED_CLK_0;

// Gated clock 1
typedef union {
	unsigned char		reg;
	struct {
		unsigned char	mft			: 1;
		unsigned char	t8			: 1;
		unsigned char	reserved	: 5;
		unsigned char	sramc		: 1;
	} bit;
} U_CMU_GATED_CLK_1;

// Gated clock 2
typedef union {
	unsigned char		reg;
	struct {
		unsigned char	rtcSapb		: 1;
		unsigned char	port		: 1;
		unsigned char	wdt			: 1;
		unsigned char	adc			: 1;
		unsigned char	remc		: 1;
		unsigned char	spi			: 1;
		unsigned char	reserved	: 2;
	} bit;
} U_CMU_GATED_CLK_2;

// CMU configuration
typedef struct {
	unsigned char		osc3On;
	unsigned char		oscSel;
	unsigned char		wakeupWait;
	unsigned char		waitTimerMode;
	unsigned char		osc3OffinSleep;
	unsigned char		osc3WaitTimer;
	unsigned char		osc3Nf;
	unsigned char		inPortNf;
	unsigned char		dsioNf;
	unsigned char		osc3ClockDiv;
	unsigned char		lcdcClockDiv;
	unsigned char		cmuClockSel;

	U_CMU_GATED_CLK_0	gatedClk0;
	U_CMU_GATED_CLK_1	gatedClk1;
	U_CMU_GATED_CLK_2	gatedClk2;

	unsigned char		usbWait;
	unsigned char		usbSnooze;
} T_CMU_CFG;


// Macro function
// For system clock control register
#define	CmuSetSysClk(x)				(CMU_SYSCLKCTL = x)
#define	CmuGetSysClk()				(CMU_SYSCLKCTL)
#define	CmuSysClkOsc3On()			(CMU_SYSCLKCTL |= CMU_SYSCLKCTL_SOSC3)
#define	CmuSysClkOsc3Off()			(CMU_SYSCLKCTL &= ~(CMU_SYSCLKCTL_SOSC3))
#define	CmuSysClkSelOsc1			(CMU_SYSCLKCTL |= CMU_SYSCLKCTL_OSCSEL)
#define	CmuSysClkSelOsc3			(CMU_SYSCLKCTL &= ~(CMU_SYSCLKCTL_OSCSEL))
#define	CmuSysClkWkupWaitInt()		(CMU_SYSCLKCTL |= CMU_SYSCLKCTL_WAKEUPET)
#define	CmuSysClkWkupNoWait()		(CMU_SYSCLKCTL &= ~(CMU_SYSCLKCTL_WAKEUPET))
#define	CmuSysClkWaitTimHigh()		(CMU_SYSCLKCTL |= CMU_SYSCLKCTL_TMHSP)
#define	CmuSysClkWaitTimNormal()	(CMU_SYSCLKCTL &= ~(CMU_SYSCLKCTL_TMHSP))
#define	CmuSysClkOsc3Stop()			(CMU_SYSCLKCTL |= CMU_SYSCLKCTL_OSC3OFF)
#define	CmuSysClkOsc3Run()			(CMU_SYSCLKCTL &= ~(CMU_SYSCLKCTL_OSC3OFF))

// For OSC3 wait timer register
#define	CmuSetOsc3WaitTimer(x)		(CMU_OSC3_WCNT = x)
#define	CmuGetOsc3WaitTimer()		(CMU_OSC3_WCNT)

// For noise filter control register
#define	CmuSetNfControl(x)			(CMU_NF = x)
#define	CmuGetNfControl(x)			(CMU_NF)
#define	CmuOsc3NfDisable()			(CMU_NF |= CMU_NF_OSC3NF)
#define	CmuOsc3NfEnable()			(CMU_NF &= ~(CMU_NF_OSC3NF))
#define	CmuInPortNfDisable()		(CMU_NF |= CMU_NF_INPORTNF)
#define	CmuInPortNfEnable()			(CMU_NF &= ~(CMU_NF_INPORTNF))
#define	CmuDsioInNfDisable()		(CMU_NF &= ~(CMU_NF_DSINNF))
#define	CmuDsioInNfEnable()			(CMU_NF |= CMU_NF_DSINNF)

// For OSC3 clock divider register
#define	CmuSetOsc3ClkDiv(x)			(CMU_OSC3DIV = x)
#define	CmuGetOsc3ClkDiv()			(CMU_OSC3DIV & CMU_OSC3DIV_OSC3DIV))

// For LCDC clock divider register
#define	CmuSetLcdcClkDiv(x)			(CMU_LCDCDIV = x)
#define	CmuGetLcdcClkDiv()			(CMU_LCDCDIV & CMU_LCDCDIV_LCDCDIV))

// For "CMU_CLK" select register
#define	CmuSetCmuClkSel(x)			(CMU_CMUCLK = x)
#define	CmuGetCmuClkSel()			(CMU_CMUCLK & CMU_CMUCLK_CMU_CLK_SEL))

// For gated clock control register 0
#define	CmuSetGatedClk0(x)			(CMU_GATEDCLK0 = x)
#define	CmuGetGatedClk0()			(CMU_GATEDCLK0)
#define	CmuGatedClk0PclkOn()		(CMU_GATEDCLK0 |= CMU_GATEDCLK0_PCLK_SOC_EN)
#define	CmuGatedClk0Off()			(CMU_GATEDCLK0 &= ~(CMU_GATEDCLK0_PCLK_SOC_EN))
#define	CmuGatedClk0LcdcOn()		(CMU_GATEDCLK0 |= CMU_GATEDCLK0_LCDC_CLK_EN)
#define	CmuGatedClk0LcdcOff()		(CMU_GATEDCLK0 &= ~(CMU_GATEDCLK0_LCDC_CLK_EN))
#define	CmuGatedClk0LcdcSapbOn()	(CMU_GATEDCLK0 |= CMU_GATEDCLK0_LCDC_SAPB_CLK_EN)
#define	CmuGatedClk0LcdcSapbOff()	(CMU_GATEDCLK0 &= ~(CMU_GATEDCLK0_LCDC_SAPB_CLK_EN))
#define	CmuGatedClk0LcdcHostIfOn()	(CMU_GATEDCLK0 |= CMU_GATEDCLK0_LCDC_HOSTIF_CLK_EN)
#define	CmuGatedClk0LcdcHostIfOff()	(CMU_GATEDCLK0 &= ~(CMU_GATEDCLK0_LCDC_HOSTIF_CLK_EN))
#define	CmuGatedClk0UsbOn()			(CMU_GATEDCLK0 |= CMU_GATEDCLK0_USB_CLK_EN)
#define	CmuGatedClk0UsbOff()		(CMU_GATEDCLK0 &= ~(CMU_GATEDCLK0_USB_CLK_EN))
#define	CmuGatedClk0UsbSapbOn()		(CMU_GATEDCLK0 |= CMU_GATEDCLK0_USB_SAPB_CLK_EN)
#define	CmuGatedClk0UsbSapbOff()	(CMU_GATEDCLK0 &= ~(CMU_GATEDCLK0_USB_SAPB_CLK_EN))
#define	CmuGatedClk0FlashcOn()		(CMU_GATEDCLK0 |= CMU_GATEDCLK0_FLASHC_CLK_EN)
#define	CmuGatedClk0FlashcOff()		(CMU_GATEDCLK0 &= ~(CMU_GATEDCLK0_FLASHC_CLK_EN))

// For gated clock control register 1
#define	CmuSetGatedClk1(x)			(CMU_GATEDCLK1 = x)
#define	CmuGetGatedClk1()			(CMU_GATEDCLK1)
#define	CmuGatedClk1MftOn()			(CMU_GATEDCLK1 |= CMU_GATEDCLK1_MFT_CLK_EN)
#define	CmuGatedClk1MftOff()		(CMU_GATEDCLK1 &= ~(CMU_GATEDCLK1_MFT_CLK_EN))
#define	CmuGatedClk1T8On()			(CMU_GATEDCLK1 |= CMU_GATEDCLK1_T8_CLK_EN)
#define	CmuGatedClk1T8Off()			(CMU_GATEDCLK1 &= ~(CMU_GATEDCLK1_T8_CLK_EN))
#define	CmuGatedClk1SramcOn(x)		(CMU_GATEDCLK1 |= CMU_GATEDCLK1_SRAMC_CLK_EN)
#define	CmuGatedClk1SramcOff()		(CMU_GATEDCLK1 &= ~(CMU_GATEDCLK1_SRAMC_CLK_EN))

// For gated clock control register 2
#define	CmuSetGatedClk2(x)			(CMU_GATEDCLK2 = x)
#define	CmuGetGatedClk2()			(CMU_GATEDCLK2)
#define	CmuGatedClk2RtcSapbOn()		(CMU_GATEDCLK2 |= CMU_GATEDCLK2_RTC_SAPB_CLK_EN)
#define	CmuGatedClk2RtcSapbOff()	(CMU_GATEDCLK2 &= ~(CMU_GATEDCLK2_RTC_SAPB_CLK_EN))
#define	CmuGatedClk2PortOn()		(CMU_GATEDCLK2 |= CMU_GATEDCLK2_PORT_CLK_EN)
#define	CmuGatedClk2PortOff()		(CMU_GATEDCLK2 &= ~(CMU_GATEDCLK2_PORT_CLK_EN))
#define	CmuGatedClk2WdtOn()			(CMU_GATEDCLK2 |= CMU_GATEDCLK2_WDT_CLK_EN)
#define	CmuGatedClk2WdtOff()		(CMU_GATEDCLK2 &= ~(CMU_GATEDCLK2_WDT_CLK_EN))
#define	CmuGatedClk2AdcOn()			(CMU_GATEDCLK2 |= CMU_GATEDCLK2_ADC_CLK_EN)
#define	CmuGatedClk2AdcOff()		(CMU_GATEDCLK2 &= ~(CMU_GATEDCLK2_ADC_CLK_EN))
#define	CmuGatedClk2RemcOn()		(CMU_GATEDCLK2 |= CMU_GATEDCLK2_REMC_CLK_EN)
#define	CmuGatedClk2RemcOff()		(CMU_GATEDCLK2 &= ~(CMU_GATEDCLK2_REMC_CLK_EN))
#define	CmuGatedClk2SpiOn()			(CMU_GATEDCLK2 |= CMU_GATEDCLK2_SPI_CLK_EN)
#define	CmuGatedClk2SpiOff()		(CMU_GATEDCLK2 &= ~(CMU_GATEDCLK2_SPI_CLK_EN))

// For USB wait control register
#define	CmuSetUsbWaitControl(x)		(CMU_USBWT = x)
#define	CmuGetUsbWaitControl(x)		(CMU_USBWT)
#define	CmuSetUsbWait(x)			(CMU_USBWT = ((CMU_USBWT & ~(CMU_USBWT_USBWT)) | (x & CMU_USBWT_USBWT)))
#define	CmuGetUsbWait()				(CMU_USBWT & CMU_USBWT_USBWT))
#define	CmuUsbSnoozeEnable()		(CMU_USBWT |= CMU_USBWT_USBSNZ)
#define	CmuUsbSnoozeDisable()		(CMU_USBWT &= ~(CMU_USBWT_USBSNZ))

// For clock control protect register
#define	CmuWrProtectEnable()		(CMU_PROTECT = 0x00)
#define	CmuWrProtectDisable()		(CMU_PROTECT = 0x96)



#ifdef	__cplusplus
extern "C" {
#endif


// SRAMC APIs
extern void CmuInit(T_CMU_CFG *pConfig);


#ifdef __cplusplus
}
#endif

#endif	// __CMU_API_H__

