/***************************************************************************
*
*	File name : main.c
*	Function  : Main function
*	Revision  :
*		07/31/2007	Nariai		First revision
*
****************************************************************************/

// Include File
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<ctype.h>
#include	"reg_801.h"
#include	"system.h"
#include	"itc_api.h"
#include	"gpio_api.h"
#include	"flashc_api.h"

// Define


// Prototype
int main(void);
static void PortInitialize(void);
static void ItcInitialize(void);
static void FlashcInitialize(void);

// Global
unsigned char	inbuff[128];
unsigned char	outbuff[128];

/***************************************************************************
	main

	description	: main function
	argument	:
	return: 	:

	input		:
	output		:

	flag		:
	global		:
***************************************************************************/
int main(void){

	char			str[256];
	unsigned long	sadd,eadd;
	unsigned short	*p;
	unsigned long	i;

	PortInitialize();
	ItcInitialize();
	FlashcInitialize();

	// CPU interrupt enable
	CpuIntEnable();

	sprintf(outbuff, "This is sample program for Internal Flash.\n");
	printf(outbuff);

	p = (unsigned short *)0xa00000;

	for(i = 0; i < 0x10000;i++){
		*p++ = i;
	}

	p = (unsigned short *)0xa00000;

	while(1){
		sprintf(outbuff, "command > ");
		printf(outbuff);
		scanf("%s", str);

		if(strcmp(str, "?") == 0 || strcmp(str, "help") == 0){
			sprintf(outbuff, "chip  : chip erase\n");
			printf(outbuff);
			sprintf(outbuff, "sec   : sector erase\n");
			printf(outbuff);
			sprintf(outbuff, "write : write data\n");
			printf(outbuff);
			sprintf(outbuff, "all   : write data\n");
			printf(outbuff);
		} else if(strcmp(str, "chip") == 0){
			if(FlashcErase(FLASHC_ERASE_CHIP, 0x20000, 0x20000) == ERROR){
				sprintf(outbuff, "Chip erase failed\n");
				printf(outbuff);
			} else {
				sprintf(outbuff, "Chip erase complete\n");
				printf(outbuff);
			}
		} else if(strcmp(str, "sec") == 0){
			sprintf(outbuff, "start address (0xXXXX)> ");
			printf(outbuff);
			scanf("%s", str);
			sscanf(str, "0x%lX", &sadd);

			sprintf(outbuff, "end address   (0xXXXX)> ");
			printf(outbuff);
			scanf("%s", str);
			sscanf(str, "0x%lX", &eadd);

			sprintf(outbuff, "start address = 0x%08lX\n", sadd);
			printf(outbuff);
			sprintf(outbuff, "end   address = 0x%08lX\n", eadd);
			printf(outbuff);

			if(FlashcErase(FLASHC_ERASE_SECTOR, sadd, eadd) == ERROR){
				sprintf(outbuff, "Sector erase failed\n");
				printf(outbuff);
			} else {
				sprintf(outbuff, "Sector erase complete\n");
				printf(outbuff);
			}

		} else if(strcmp(str, "write") == 0){
			sprintf(outbuff, "write address (0xXXXX)> ");
			printf(outbuff);
			scanf("%s", str);
			sscanf(str, "0x%lX", &sadd);
			sprintf(outbuff, "address = 0x%08lX\n", sadd);
			printf(outbuff);

			if(FlashcWrite(sadd, 0x400, (unsigned short *)p) == ERROR){
				sprintf(outbuff, "Sector write failed\n");
				printf(outbuff);
			} else {
				sprintf(outbuff, "Sector write complete\n");
				printf(outbuff);
			}
		} else if(strcmp(str, "all") == 0){

			if(FlashcWrite(0x20000, 0x20000, (unsigned short *)p) == ERROR){
				sprintf(outbuff, "All write failed\n");
				printf(outbuff);
			} else {
				sprintf(outbuff, "All write complete\n");
				printf(outbuff);
			}
		}
	}

	return 0;

}


/***************************************************************************
	PortInitialize

	description	: Init. Port
	argument	: None
	return: 	: None

	input		:
	output		:

	flag		: None
	global		: None
***************************************************************************/
static void PortInitialize(void){

}

/***************************************************************************
	ItcInitialize

	description	: Init. ITC
	argument	: None
	return: 	: None

	input		:
	output		:

	flag		: None
	global		: None
***************************************************************************/
static void ItcInitialize(void){

	T_ITC_CFG	itcConfig;

	itcConfig.addIntEn.bit.mft			= ITC_INT_DISABLE;
	itcConfig.addIntEn.bit.adcRange		= ITC_INT_DISABLE;
	itcConfig.addIntEn.bit.adcEnd		= ITC_INT_DISABLE;
	itcConfig.addIntEn.bit.rtc			= ITC_INT_DISABLE;
	itcConfig.addIntEn.bit.t8Ch0		= ITC_INT_DISABLE;
	itcConfig.addIntEn.bit.t8Ch1		= ITC_INT_DISABLE;
	itcConfig.addIntEn.bit.t8Ch2		= ITC_INT_DISABLE;
	itcConfig.addIntEn.bit.t8Ch3		= ITC_INT_DISABLE;
	itcConfig.addIntEn.bit.lcdc			= ITC_INT_DISABLE;
	itcConfig.addIntEn.bit.espi			= ITC_INT_DISABLE;
	itcConfig.addIntEn.bit.usb			= ITC_INT_DISABLE;
	itcConfig.addIntEn.bit.i2sTran		= ITC_INT_DISABLE;
	itcConfig.addIntEn.bit.i2sRcv		= ITC_INT_DISABLE;
	itcConfig.addIntEn.bit.remc			= ITC_INT_DISABLE;

	itcConfig.addIntLv0.bit.mft			= 0;
	itcConfig.addIntLv1.bit.adcRange	= 0;
	itcConfig.addIntLv1.bit.adcEnd		= 0;
	itcConfig.addIntLv2.bit.rtc			= 0;
	itcConfig.addIntLv2.bit.t8Ch0		= 0;
	itcConfig.addIntLv3.bit.t8Ch1		= 0;
	itcConfig.addIntLv3.bit.t8Ch2		= 0;
	itcConfig.addIntLv4.bit.t8Ch3		= 0;
	itcConfig.addIntLv4.bit.lcdc		= 0;
	itcConfig.addIntLv5.bit.espi		= 0;
	itcConfig.addIntLv5.bit.usb			= 0;
	itcConfig.addIntLv6.bit.i2sTran		= 0;
	itcConfig.addIntLv6.bit.i2sRcv		= 0;
	itcConfig.addIntLv7.bit.remc		= 0;

	itcConfig.intEn.bit.port0			= ITC_INT_DISABLE;
	itcConfig.intEn.bit.port1			= ITC_INT_DISABLE;
	itcConfig.intEn.bit.port2			= ITC_INT_DISABLE;
	itcConfig.intEn.bit.port3			= ITC_INT_DISABLE;
	itcConfig.intEn.bit.port4			= ITC_INT_DISABLE;
	itcConfig.intEn.bit.port5			= ITC_INT_DISABLE;
	itcConfig.intEn.bit.port6			= ITC_INT_DISABLE;
	itcConfig.intEn.bit.port7			= ITC_INT_DISABLE;
	itcConfig.intEn.bit.clgT16U			= ITC_INT_DISABLE;
	itcConfig.intEn.bit.clgT8S			= ITC_INT_DISABLE;
	itcConfig.intEn.bit.clgT8I			= ITC_INT_DISABLE;
	itcConfig.intEn.bit.uart			= ITC_INT_DISABLE;
	itcConfig.intEn.bit.spi				= ITC_INT_DISABLE;
	itcConfig.intEn.bit.i2c				= ITC_INT_DISABLE;

	itcConfig.extIntLv0.bit.port0		= 3;
	itcConfig.extIntLv0.bit.port0_TM	= ITC_TM_LEVEL;
	itcConfig.extIntLv0.bit.port0_TP	= ITC_TP_POSITIVE;
	itcConfig.extIntLv0.bit.port1		= 3;
	itcConfig.extIntLv0.bit.port1_TM	= ITC_TM_LEVEL;
	itcConfig.extIntLv0.bit.port1_TP	= ITC_TP_POSITIVE;
	itcConfig.extIntLv1.bit.port2		= 3;
	itcConfig.extIntLv1.bit.port2_TM	= ITC_TM_LEVEL;
	itcConfig.extIntLv1.bit.port2_TP	= ITC_TP_POSITIVE;
	itcConfig.extIntLv1.bit.port3		= 3;
	itcConfig.extIntLv1.bit.port3_TM	= ITC_TM_LEVEL;
	itcConfig.extIntLv1.bit.port3_TP	= ITC_TP_POSITIVE;
	itcConfig.extIntLv2.bit.port4		= 3;
	itcConfig.extIntLv2.bit.port4_TM	= ITC_TM_LEVEL;
	itcConfig.extIntLv2.bit.port4_TP	= ITC_TP_POSITIVE;
	itcConfig.extIntLv2.bit.port5		= 3;
	itcConfig.extIntLv2.bit.port5_TM	= ITC_TM_LEVEL;
	itcConfig.extIntLv2.bit.port5_TP	= ITC_TP_POSITIVE;
	itcConfig.extIntLv3.bit.port6		= 3;
	itcConfig.extIntLv3.bit.port6_TM	= ITC_TM_LEVEL;
	itcConfig.extIntLv3.bit.port6_TP	= ITC_TP_POSITIVE;
	itcConfig.extIntLv3.bit.port7		= 3;
	itcConfig.extIntLv3.bit.port7_TM	= ITC_TM_LEVEL;
	itcConfig.extIntLv3.bit.port7_TP	= ITC_TP_POSITIVE;

	itcConfig.inIntLv0.bit.clgT16U		= 3;
	itcConfig.inIntLv1.bit.clgT8S		= 3;
	itcConfig.inIntLv1.bit.clgT8I		= 3;
	itcConfig.inIntLv2.bit.uart			= 3;
	itcConfig.inIntLv3.bit.spi			= 3;
	itcConfig.inIntLv3.bit.i2c			= 3;

	itcConfig.funcEnable				= ITC_FUNC_ENABLE;

	ItcInit(&itcConfig);

	return;

}
/***************************************************************************
	FlashcInitialize

	description	: Initialize for Internal flashc
	argument	: None
	return: 	: None

***************************************************************************/
static void FlashcInitialize(void){

	T_FLASHC_CFG	config;

	config.flashWait = 1;			// Access wait value
	config.TTBR = 0xFFFFFFFF;		// Trap Table base address

	FlashcInit(&config);
}
